# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""
from collective.faq import _
from plone.app.textfield import RichText
from plone.supermodel import model
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ICollectivefaqCoreLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IFAQ(model.Schema):
    """FAQ content type interface."""


class IFAQItem(Interface):
    """FAQ Item content type interface."""

    title = schema.TextLine(
        title=_(u"Question"),
        required=True,
    )

    detailed_question = schema.Text(
        title=_(u"Detailed Question"),
        description=_(u"More details on the question, if not evident from the title"),
        required=False,
    )

    answer = RichText(
        title=_(u"Answer"),
        description=_(u"Meaningful sentences that explains the answer"),
        required=False,
    )



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_ID
from zope.component import queryUtility
from zope.component import createObject
from plone.app.testing import setRoles
from plone.dexterity.interfaces import IDexterityFTI
from plone import api

from collective.faq.testing import COLLECTIVEFAQ_CORE_INTEGRATION_TESTING  # noqa
from collective.faq.interfaces import IFAQItem

import unittest


class FAQItemIntegrationTest(unittest.TestCase):

    layer = COLLECTIVEFAQ_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer = api.portal.get_tool("portal_quickinstaller")
        fti = queryUtility(IDexterityFTI, name="FAQ Item")
        fti.global_allow = True

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="FAQ Item")
        schema = fti.lookupSchema()
        self.assertEqual(IFAQItem, schema)

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="FAQ Item")
        self.assertTrue(fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="FAQ Item")
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(IFAQItem.providedBy(obj))

    def test_adding(self):
        self.portal.invokeFactory("FAQ Item", "faqitem")
        self.assertTrue(IFAQItem.providedBy(self.portal["faqitem"]))



# -*- coding: utf-8 -*-
from plone import api
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2

import collective.faq


class CollectivefaqCoreLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.faq)

    def setUpPloneSite(self, portal):
        setRoles(portal, TEST_USER_ID, ["Manager"])
        login(portal, TEST_USER_NAME)
        api.content.create(
            type="Document", id="front-page", title="Welcome", container=portal
        )
        logout()
        applyProfile(portal, "collective.faq:default")


COLLECTIVEFAQ_CORE_FIXTURE = CollectivefaqCoreLayer()


COLLECTIVEFAQ_CORE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVEFAQ_CORE_FIXTURE,),
    name="CollectivefaqCoreLayer:IntegrationTesting",
)


COLLECTIVEFAQ_CORE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVEFAQ_CORE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="CollectivefaqCoreLayer:FunctionalTesting",
)


COLLECTIVEFAQ_CORE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVEFAQ_CORE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="CollectivefaqCoreLayer:AcceptanceTesting",
)



# -*- coding: utf-8 -*-
from collective.faq.interfaces import IFAQ
from collective.faq.interfaces import IFAQItem
from plone import api
from plone.dexterity.browser.view import DefaultView


class FAQView(DefaultView):
    """Defaul view for FAQ content type."""

    def questions(self, faq=None):
        provides = [IFAQItem]
        if not faq:
            faq = self.context
            provides += [IFAQ]
        return api.content.find(
            context=faq,
            depth=1,
            object_provides=provides,
            sort_on="getObjPositionInParent",
        )

    def nested_item(self, obj):
        return IFAQ.providedBy(obj)


class FAQItemView(DefaultView):
    """Defaul view for FAQ Item content type."""



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.faq.testing import COLLECTIVEFAQ_CORE_INTEGRATION_TESTING  # noqa
from plone import api

try:
    from Products.CMFPlone.utils import get_installer
except ImportError:  # Plone < 5.1
    HAS_INSTALLER = False
else:
    HAS_INSTALLER = True

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.faq is properly installed."""

    layer = COLLECTIVEFAQ_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if HAS_INSTALLER:
            self.installer = get_installer(self.portal)
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if collective.faq is installed."""
        if HAS_INSTALLER:
            self.assertTrue(self.installer.is_product_installed("collective.faq"))
        else:
            self.assertTrue(self.installer.isProductInstalled("collective.faq"))

    def test_browserlayer(self):
        """Test that ICollectivefaqCoreLayer is registered."""
        from collective.faq.interfaces import ICollectivefaqCoreLayer
        from plone.browserlayer import utils

        self.assertIn(ICollectivefaqCoreLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVEFAQ_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if HAS_INSTALLER:
            self.installer = get_installer(self.portal)
            self.installer.uninstall_product("collective.faq")
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
            self.installer.uninstallProducts(["collective.faq"])

    def test_product_uninstalled(self):
        """Test if collective.faq is cleanly uninstalled."""
        if HAS_INSTALLER:
            self.assertFalse(self.installer.is_product_installed("collective.faq"))
        else:
            self.assertFalse(self.installer.isProductInstalled("collective.faq"))

    def test_browserlayer_removed(self):
        """Test that ICollectivefaqCoreLayer is removed."""
        from collective.faq.interfaces import ICollectivefaqCoreLayer
        from plone.browserlayer import utils

        self.assertNotIn(ICollectivefaqCoreLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_ID
from zope.component import queryUtility
from zope.component import createObject
from plone.app.testing import setRoles
from plone.dexterity.interfaces import IDexterityFTI
from Products.CMFPlone.utils import get_installer

from collective.faq.testing import COLLECTIVEFAQ_CORE_INTEGRATION_TESTING  # noqa
from collective.faq.interfaces import IFAQ

import unittest


class FAQIntegrationTest(unittest.TestCase):

    layer = COLLECTIVEFAQ_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer = get_installer(self.portal, self.layer["request"])
        fti = queryUtility(IDexterityFTI, name="FAQ")
        fti.global_allow = True

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="FAQ")
        schema = fti.lookupSchema()
        self.assertEqual(IFAQ, schema)

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="FAQ")
        self.assertTrue(fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="FAQ")
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(IFAQ.providedBy(obj))

    def test_adding(self):
        self.portal.invokeFactory("FAQ", "faq")
        self.assertTrue(IFAQ.providedBy(self.portal["faq"]))



# -*- coding: utf-8 -*-
from .interfaces import IFAQ
from .interfaces import IFAQItem
from plone.dexterity.content import Container
from plone.dexterity.content import Item
from zope.interface import implementer


@implementer(IFAQ)
class FAQ(Container):
    """FAQ content type."""


@implementer(IFAQItem)
class FAQItem(Item):
    """FAQ Item content type."""



# -*- coding: utf-8 -*-
"""Installer for the collective.faq package."""

from setuptools import find_packages
from setuptools import setup


long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="collective.faq",
    version="2.0.0",
    description="Plone addon package for managing FAQ sections",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone",
    author="kitconcept GmbH",
    author_email="info@kitconcept.com",
    url="https://github.com/collective/collective.faq",
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["collective"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.api",
        "plone.app.content",
        "plone.app.contenttypes",
        "plone.app.dexterity",
        "plone.app.lockingbehavior",
        "plone.app.relationfield",
        "plone.app.versioningbehavior",
        "Products.GenericSetup>=1.8.2",
        "setuptools",
        "z3c.jbot",
        # 'plone.restapi',
        # 'Products.PloneFormGen',
        # 'plone.formwidget.captcha',
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            # Plone KGS does not use this version, because it would break
            # Remove if your package shall be part of coredev.
            # plone_coredev tests as of 2016-04-01.
            "plone.testing>=5.0.0",
            "plone.app.contenttypes",
            "plone.app.robotframework[debug]",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
